SystemOrganization addCategory: #'Rest-Tests'!
SystemOrganization addCategory: #'Rest-Tests-Server'!
SystemOrganization addCategory: #'Rest-Tests-Fixtures'!
SystemOrganization addCategory: #'Rest-Tests-Client'!

RestHttpHandler subclass: #RestPerformHttpHandler
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Rest-Tests-Fixtures'!

!RestPerformHttpHandler methodsFor: 'request processing' stamp: 'cs 2/19/2009 17:51'!
process: anHTTPRequest
	"Takes first element in path as classname and sends message from third path element to that class."

	|pc class selector result|
	pc := anHTTPRequest uri identifierPath.
	class := pc second asSymbol.
	selector := pc third asSymbol.
	
	Smalltalk at: class ifPresent: [:c | 
		(c class allSelectors includes: selector) ifTrue: [
			result := c perform: selector]].

	^ HTTPResponse ok
		entity: result asString;
		yourself! !

Object subclass: #TestObject
	instanceVariableNames: 'firstName lastName id'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Rest-Tests-Fixtures'!

!TestObject class methodsFor: 'instance creation' stamp: 'topa 2/25/2009 11:32'!
all
	^ OrderedCollection with: self newPrepared.! !

!TestObject class methodsFor: 'instance creation' stamp: 'topa 2/25/2009 11:31'!
newPrepared

	^ self new
		firstName: 'Test';
		lastName: 'Project';
		id: 1! !

!TestObject methodsFor: 'accessing' stamp: 'topa 2/25/2009 11:31'!
firstName
	^ firstName! !

!TestObject methodsFor: 'accessing' stamp: 'topa 2/25/2009 11:31'!
firstName: anObject
	firstName := anObject! !

!TestObject methodsFor: 'accessing' stamp: 'topa 2/25/2009 11:31'!
id
	^ id! !

!TestObject methodsFor: 'accessing' stamp: 'topa 2/25/2009 11:31'!
id: anObject
	id := anObject! !

!TestObject methodsFor: 'printing' stamp: 'topa 2/25/2009 11:52'!
jsonWriteOn: aStream

	(Dictionary newFrom: (
		#(id firstName lastName) collect: [ :aSymbol |
		aSymbol -> (self perform: aSymbol)])) jsonWriteOn: aStream! !

!TestObject methodsFor: 'accessing' stamp: 'topa 2/25/2009 11:31'!
lastName
	^ lastName! !

!TestObject methodsFor: 'accessing' stamp: 'topa 2/25/2009 11:31'!
lastName: anObject
	lastName := anObject! !

RestResourceHttpHandler subclass: #RestMockResourceHttpHandler
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Rest-Tests-Fixtures'!

!RestMockResourceHttpHandler class methodsFor: 'as yet unclassified' stamp: 'mfp 2/24/2009 14:30'!
rootResources
	^ Dictionary newFrom:
		{'tests' -> TestObject }! !

!RestMockResourceHttpHandler class methodsFor: 'as yet unclassified' stamp: 'mfp 2/24/2009 14:31'!
uriMapping

	^ Dictionary newFrom:{
		TestObject -> [:test| '/tests/{1}' format: {test id}]}! !

!Dictionary methodsFor: '*rest-tests' stamp: 'mt 2/16/2009 10:28'!
keysAndValuesCollect: aBlock 
	| newDict |
	newDict := Dictionary new.
	self keysAndValuesDo: [:key :value | newDict add: (aBlock value: key value: value)].
	^ newDict! !

TestCase subclass: #RestSocketTestCase
	instanceVariableNames: 'site'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Rest-Tests-Client'!

!RestSocketTestCase class methodsFor: 'accessing' stamp: 'cs 2/19/2009 20:47'!
httpUrl
	^ 'http://www.google.com' asUrl! !

!RestSocketTestCase class methodsFor: 'accessing' stamp: 'topa 2/19/2009 19:10'!
url
	^ 'https://www.verisign.com/favicon.ico' asUrl! !

!RestSocketTestCase methodsFor: 'tests' stamp: 'mig 2/23/2009 16:45'!
testDefaultReturn

	self
		should: ((RestSocket    getDocumentFromUrl: self class httpUrl request: '') isKindOf: RestResponse);
		should: ((RestSocket    getDocumentFromUrl: self class url request: '') isKindOf: RestResponse);
		should: ((RestSocket    putDocumentFromUrl: self class url request: '' content: 'foo') isKindOf: RestResponse);
		should: ((RestSocket   postDocumentFromUrl: self class url request: '' content: 'bar') isKindOf: RestResponse);
		should: ((RestSocket deleteDocumentFromUrl: self class url request: '') isKindOf: RestResponse).! !

TestCase subclass: #RestTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Rest-Tests-Server'!

RestTest subclass: #RestHttpServiceTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Rest-Tests-Server'!

!RestHttpServiceTest methodsFor: 'running' stamp: 'topa 2/25/2009 13:28'!
setUp
	"avoid upcall"! !

!RestHttpServiceTest methodsFor: 'tests' stamp: 'cs 2/24/2009 18:27'!
testStartAndStop
	
	RestHttpService start.
	(Delay forMilliseconds: 100) wait.
	
	self
		should: (RestHttpService resource isKindOf: RestHttpService);
		should: RestHttpService isRunning.
	
	RestHttpService stop.
	(Delay forMilliseconds: 100) wait.
	
	self
		should: (RestHttpService resource isNil);
		shouldnt: RestHttpService isRunning.! !

RestTest subclass: #RestObjectTest
	instanceVariableNames: 'testProject testUser testTask testObject'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Rest-Tests-Client'!

!RestObjectTest methodsFor: 'running' stamp: 'topa 2/25/2009 11:21'!
setUp
	"Install a handler for http service. Add a testObject to the database."

	super setUp.
	RestHttpService resource installHandler: RestMockResourceHttpHandler on: '/'.
	testObject := TestObject newPrepared.
! !

!RestObjectTest methodsFor: 'running' stamp: 'mfp 2/24/2009 22:37'!
tearDown
	super tearDown.
	testObject := nil.! !

!RestObjectTest methodsFor: 'tests' stamp: 'topa 2/25/2009 14:37'!
testGetRestObject
	| sut uri |
	
	uri := SwazooURI fromString: RestHttpService site currentUrl, 'tests/', testObject id asString.
	sut := RestObject newFromURI: uri.
	
	self 
		should: sut firstName = testObject firstName;
		should: sut lastName = testObject lastName;
		should: sut id = testObject id.! !

RestTest subclass: #RestPerformTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Rest-Tests-Server'!

!RestPerformTest methodsFor: 'running' stamp: 'cs 2/24/2009 17:36'!
setUp
	super setUp.
	RestHttpService resource
		installHandler: RestPerformHttpHandler
		on: '/'! !

!RestPerformTest methodsFor: 'tests' stamp: 'mt 2/23/2009 12:18'!
testPort
	| response |
	response := self get: 'RestHttpService/port'.
	self should: response content = RestHttpService port asString.! !

!RestTest class methodsFor: 'fixture-data' stamp: 'topa 2/25/2009 11:16'!
host
	'localhost'! !

!RestTest class methodsFor: 'fixture-data' stamp: 'topa 2/25/2009 11:16'!
ip
	'127.0.0.1'! !

!RestTest class methodsFor: 'testing' stamp: 'cs 2/15/2009 14:14'!
isAbstract
	^ self name = 'RestTest'! !

!RestTest class methodsFor: 'fixture-data' stamp: 'topa 2/25/2009 11:15'!
port
	^ 8082! !

!RestTest methodsFor: 'helping' stamp: 'mig 2/23/2009 16:45'!
delete: aPath
	"Send an HTTP request with DELETE to localhost at the port of the (singleton) rest HTTP service."
	
	^ RestSocket
		deleteDocumentFromUrl: (('http://localhost:{1}/{2}' format: {RestHttpService port. aPath}) asUrl)
		request: ''! !

!RestTest methodsFor: 'helping' stamp: 'topa 2/25/2009 13:16'!
get: aPath
	"Send an HTTP request with GET to localhost at the port of the (singleton) rest HTTP service."

	^ RestSocket
		getDocumentFromUrl: ('http://localhost:{1}/{2}' format: {RestHttpService port. aPath}) asUrl
		request: ''! !

!RestTest methodsFor: 'helping' stamp: 'mt 2/18/2009 15:09'!
jsonDataFrom: someFields
	"Create a string for all fields in the dictionary in JSON."
	
	| stream |
	stream := TextStream with: ''.
	(Dictionary newFrom: someFields) jsonWriteOn: stream.	
	^ stream contents! !

!RestTest methodsFor: 'helping' stamp: 'mig 2/23/2009 16:45'!
post: aPath content: aContent
	"Send an HTTP request with POST to localhost at the port of the (singleton) rest HTTP service. Append the given fields to the request body."

	^ RestSocket
		postDocumentFromUrl: (('http://localhost:{1}/{2}' format: {RestHttpService port. aPath}) asUrl)
		request: ''
		content: aContent! !

!RestTest methodsFor: 'helping' stamp: 'mig 2/23/2009 16:45'!
put: aPath content: aContent
	"Send an HTTP request with PUT to localhost at the port of the (singleton) rest HTTP service. Append the given fields to the request body."
	
	^ RestSocket
		putDocumentFromUrl: (('http://localhost:{1}/{2}' format: {RestHttpService port. aPath}) asUrl)
		request: ''
		content: aContent! !

!RestTest methodsFor: 'running' stamp: 'topa 2/25/2009 11:18'!
setUp
	RestHttpService startOn: self class port.
! !

!RestTest methodsFor: 'running' stamp: 'cs 2/15/2009 14:27'!
tearDown
	RestHttpService stop.! !
